
// Asounding Audio Engine for iOS
// Copyright (c) 2011  Michel Fortin
//
// You must obtain a comercial license for using this code in software
// distributed not under the GNU GPL. See <http://michelf.com/asounding/>.
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.
//
// As a special exception to the GPL, you may distribute a compiled 
// application including this file on the Apple App Store for iOS without 
// having to redistribute the digital keys needed to reproduce the digital 
// signature needed to run an application on an iOS device. This exception 
// does not invalidate any other requirements put on you by the GPL.

#import <Foundation/Foundation.h>
#import <OpenAL/al.h>


/*!
 @abstract   A sound playable in the background by a sound source.
 */
@interface ASNSound : NSObject
{
	ALuint					buffer;
	void*					data;
}

/*! @group  Creating a Sound Object */
/*!
 @abstract   Create a new sound object from the given URL.
 @param      url  URL to read the sound from.
 @discussion The selected audio file must be in a uncompressed format 
 understood by CoreAudio. This includes AIFF, WAV, and CAF files with 
 uncompressed data.
 */
+ (id)soundWithContentOfURL:(NSURL *)url;

/*! @group  Initializing a Sound Object */
/*!
 @abstract   Initialize sound object from the given URL.
 @param      url  URL to read the sound from.
 @discussion The selected audio file must be in a uncompressed format 
 understood by  CoreAudio. This includes AIFF, WAV, and CAF files with 
 uncompressed data.
 */
- (id)initWithContentOfURL:(NSURL *)url;

// For internal use
- (ALuint)alBuffer;

@end
