
// Asounding Audio Engine for iOS
// Copyright (c) 2011  Michel Fortin
//
// You must obtain a comercial license for using this code in software
// distributed not under the GNU GPL. See <http://michelf.com/asounding/>.
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.
//
// As a special exception to the GPL, you may distribute a compiled 
// application including this file on the Apple App Store for iOS without 
// having to redistribute the digital keys needed to reproduce the digital 
// signature needed to run an application on an iOS device. This exception 
// does not invalidate any other requirements put on you by the GPL.

#import "ASNSound.h"
#import "ASNOpenALSupport.h"


@implementation ASNSound

+ (id)soundWithContentOfURL:(NSURL *)soundFile {
	return [[[self alloc] initWithContentOfURL:soundFile] autorelease];
}

- (id)initWithContentOfURL:(NSURL *)soundFile {
	ALenum  error = AL_NO_ERROR;
	ALenum  format;
	ALsizei size;
	ALsizei freq;
	
	// get some audio data from a wave file
	if (soundFile)
	{
		// Create some OpenAL Buffer Objects
		alGenBuffers(1, &buffer);
		if((error = alGetError()) != AL_NO_ERROR) {
			NSLog(@"Error Generating Buffers: %x", error);
			exit(1);
		}
		
		data = ASNGetOpenALAudioData((CFURLRef)soundFile, &size, &format, &freq);
		
		if((error = alGetError()) != AL_NO_ERROR) {
			NSLog(@"error loading sound: %x\n", error);
			return nil;
		}
		
		// use the static buffer data API
		ASNBufferDataStaticProc(buffer, format, data, size, freq);
		
		if((error = alGetError()) != AL_NO_ERROR) {
			NSLog(@"error attaching audio to buffer: %x\n", error);
		}		
	}
	else
		NSLog(@"Could not find file!\n");

	return self;
}

- (ALuint)alBuffer {
	return buffer;
}

@end
