
// Asounding Audio Engine for iOS
// Copyright (c) 2011  Michel Fortin
//
// You must obtain a comercial license for using this code in software
// distributed not under the GNU GPL. See <http://michelf.com/asounding/>.
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.
//
// As a special exception to the GPL, you may distribute a compiled 
// application including this file on the Apple App Store for iOS without 
// having to redistribute the digital keys needed to reproduce the digital 
// signature needed to run an application on an iOS device. This exception 
// does not invalidate any other requirements put on you by the GPL.

#import "ASNSource.h"
#import "ASNSound.h"
#import "ASNEngine.h"


@implementation ASNSource

- (id)init
{
	ALenum error = AL_NO_ERROR;
	alGetError(); // Clear the error
	
	// Create some OpenAL Source Objects
	alGenSources(1, &source);
	if((error = alGetError()) != AL_NO_ERROR) 
	{
		NSLog(@"Error generating sources! %x\n", error);
		exit(1);
	}
	
	engine = nil;
	[[ASNEngine sharedEngine] addSource:self];
    
	pitch = 1.0;
	
	return self;
}

- (void)dealloc {
	[[ASNEngine sharedEngine] removeSource:self];
	[super dealloc];
}

@dynamic looping;

- (BOOL)looping {
	return looping;
}

- (void)setLooping:(BOOL)flag {
	flag = flag ? AL_TRUE : AL_FALSE;
	if (flag != looping)
	{
		looping = flag;
		alSourcei(source, AL_LOOPING, looping);
	}
}

- (void)play {
	if (engine)
		alSourcePlay(source);
	
	ALenum error;
	if((error = alGetError()) != AL_NO_ERROR) {
		NSLog(@"error playing source: %x\n", error);
	}
}

- (void)pause {
	alSourcePause(source);
	
	ALenum error;
	if((error = alGetError()) != AL_NO_ERROR) {
		NSLog(@"error pausing source: %x\n", error);
	}
}

- (void)stop {
	alSourceStop(source);
	
	ALenum error;
	if((error = alGetError()) != AL_NO_ERROR) {
		NSLog(@"error stopping source: %x\n", error);
	}
}

- (void)rewind {
	alSourceRewind(source);
	
	ALenum error;
	if((error = alGetError()) != AL_NO_ERROR) {
		NSLog(@"error rewinding source: %x\n", error);
	}
}


@dynamic pitch;

- (ALfloat)pitch {
	return pitch;
}

- (void)setPitch:(ALfloat)aPitch {
	pitch = aPitch;
	alSourcef(source, AL_PITCH, pitch);
}


@dynamic sound;

- (ASNSound *)sound {
	return sound;
}

- (void)setSound:(ASNSound *)aSound {
	[sound release];
	sound = [aSound retain];
	
	ALenum error = AL_NO_ERROR;
	alGetError(); // clear previous error
	
	// attach OpenAL Buffer to OpenAL Source
	alSourcei(source, AL_BUFFER, [aSound alBuffer]);
	if ((error = alGetError()) != AL_NO_ERROR) {
		NSLog(@"Error attaching buffer to source: %x\n", error);
	}
}

@synthesize engine;

- (void)setEngine:(ASNEngine *)newEngine {
	engine = newEngine;
}

@end
