
// Demo for Asounding Audio Engine for iOS
//
// You must obtain a comercial license for using this code in software
// distributed not under the GNU GPL. See <http://michelf.com/asounding/>.
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.
//
// As a special exception to the GPL, you may distribute a compiled 
// application including this file on the Apple App Store for iOS without 
// having to redistribute the digital keys needed to reproduce the digital 
// signature needed to run an application on an iOS device. This exception 
// does not invalidate any other requirements put on you by the GPL.

#import "MusicTabController.h"


@implementation MusicTabController

- (void)loadView {
	[super loadView];
	[self updateControls];
}

- (IBAction)playMusic1 {
	if (music1 == nil) {
		// not yet loaded
		NSURL *musicURL = [[NSBundle mainBundle] URLForResource:@"return_to_home" withExtension:@"m4a"];
		music1 = [[ASNMusic alloc] initWithContentsOfURL:musicURL error:nil];
		music1.numberOfLoops = -1; // infinite looping
	}
	ASNEngine *engine = [ASNEngine sharedEngine];
	engine.music = music1;
	[engine playMusic];
	
	[self updateControls];
}
- (IBAction)playMusic2 {
	if (music2 == nil) {
		// not yet loaded
		NSURL *musicURL = [[NSBundle mainBundle] URLForResource:@"just_over_the_horizon" withExtension:@"m4a"];
		music2 = [[ASNMusic alloc] initWithContentsOfURL:musicURL error:nil];
		music2.numberOfLoops = -1; // infinite looping
	}
	ASNEngine *engine = [ASNEngine sharedEngine];
	engine.music = music2;
	[engine playMusic];
	
	[self updateControls];
}

- (IBAction)volumeChange {
	[ASNEngine sharedEngine].musicVolume = volumeSlider.value;
}

- (IBAction)pauseMusic {
	[[ASNEngine sharedEngine] pauseMusic];
	[self updateControls];
}
- (IBAction)resumeMusic {
	[[ASNEngine sharedEngine] playMusic];
	[self updateControls];
}
- (IBAction)stopMusic {
	[[ASNEngine sharedEngine] stopMusic];
	[self updateControls];
}
- (IBAction)fadeOutMusic {
	[[ASNEngine sharedEngine] fadeMusicWithin:5.0]; // 5-second fade out
	fadeOutButton.enabled = NO;
	[NSTimer scheduledTimerWithTimeInterval:5.0 target:self selector:@selector(updateControls) userInfo:nil repeats:NO];
}

- (IBAction)stopIpodChange {
	[ASNEngine sharedEngine].iPodPlaybackDisabled = stopIpodSwitch.on;
}
- (IBAction)enableMusicChange {
	[ASNEngine sharedEngine].musicEnabled = enableMusicSwitch.on;
}

- (void)updateControls {
	ASNEngine *engine = [ASNEngine sharedEngine];
	
	volumeSlider.value = engine.musicVolume;
	
	stopIpodSwitch.on = engine.iPodPlaybackDisabled;
	enableMusicSwitch.on = engine.musicEnabled;
	
	BOOL playing = engine.musicPlaying;
	BOOL paused = engine.musicPaused;
	pauseButton.enabled = playing;
	resumeButton.enabled = paused;
	stopButton.enabled = playing;
	fadeOutButton.enabled = playing;
}


@end
